// Copyright (c) 2001, 2002  Per M.A. Bothner and Brainfood Inc.
// This is free software;  for terms and warranty disclaimer see ./COPYING.

package gnu.kawa.xml;
import gnu.mapping.*;
import gnu.kawa.servlet.ServletCallContext;
import gnu.lists.*;
import gnu.xml.*;

import java.net.MalformedURLException;
import java.net.URL;
import gnu.text.*;

/** Implement the XQuery function 'document'. */

public class Document extends Procedure1
{
  public static final Document document = new Document();

  static final String DIR_SEPARATOR = "/";


  /** 
   * 
   * If there is no protocol specified, then
   *   if the xql is not a servlet, pre-pend "file:" 
   *   otherwise assume that the fileName is a relative reference in the
   *     web application contextor Servlet context and return a URL. 
   * 
   */

  public static URL makeURL(String fileName)
    throws java.net.MalformedURLException
  {
    int len = fileName.length();
    boolean seenProto = false;
    for (int i = 0;  i < len;  i++)
      {
	char ch = fileName.charAt(i);
	if (ch == ':')
	  {
	    seenProto = true;
	    break;
	  }
	if (! Character.isLetter(ch))
	  break;
      }
    if (! seenProto)
      {
        return getUrlForRelativePath( fileName );
      }
    else
      {
        return new URL(fileName);
      }
  }
  
  protected static URL getUrlForRelativePath( String fileName ) 
    throws MalformedURLException
  {
    CallContext ctx = CallContext.getInstance();
    // if the xql is a servlet, then assume that the fileName is a relative reference in the
    // web application context
    if (ctx instanceof ServletCallContext)
    {
      ServletCallContext sctx = (ServletCallContext) ctx;
      URL url = null;
      if (! fileName.startsWith( DIR_SEPARATOR ))
        {
          String spath = sctx.request.getServletPath();
          int dirIdx = spath.lastIndexOf( DIR_SEPARATOR ); 
          if (dirIdx < 0) dirIdx = 0;
          StringBuffer fullUrl = sctx.request.getRequestURL();
          int baseUrlEnd = fullUrl.indexOf(spath) + dirIdx + 1;
          fullUrl.delete( baseUrlEnd, fullUrl.length() );
          fileName = fullUrl.append( fileName ).toString();
        }
      url = new URL( fileName );
      return url;
    }
    // otherwise assume it is a file reference
    else
    {
      return new URL("file:" + fileName);
    }
    
  }

  public static void parse (Object name, Consumer out) throws Throwable
  {
    URL url = name instanceof URL ? (URL) name : makeURL(name.toString());
    SourceMessages messages = new SourceMessages();
    XMLParser parser  
      = new XMLParser(url,
		      new NamespaceResolver(out),
		      messages);
    out.beginDocument();
    parser.parse(); 
    if (messages.seenErrors())
      throw new SyntaxException(messages);
    out.endDocument();
  }

  public static TreeList parse (Object url) throws Throwable
  {
    TreeList doc = new TreeList();
    parse(url, doc);
    return doc;
  }

  public Object apply1 (Object arg1) throws Throwable
  {
    return parse(arg1.toString());
  }

  public void apply (CallContext ctx) throws Throwable
  {
    String fileName = ctx.getNextArg().toString();
    parse(makeURL(fileName), ctx.consumer);
  }
}
