/*
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */ 
 
package gnu.kawa.servlet;
   
import gnu.expr.Compilation;
import gnu.mapping.CallContext;
import gnu.mapping.InPort;
import gnu.xml.XMLPrinter;
import gnu.xquery.lang.XQuery;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.util.Enumeration;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;


/**
 * The Xql servlet interpreter
 *
 * This servlet is responsible for reading and interpeting XQL files
 * using the QEXO GNU library.
 * 
 * The implementation borrows ideas from Apache Jakarta Tomcat Jasper.
 * 
 * @author Ivelin Ivanov
 * 
 */
public class XqlServlet extends KawaServlet {

    private ServletContext context;
    private ServletConfig config;

    static 
    {
        XQuery.registerEnvironment();
        Compilation.generateServletDefault = true;
    }

    public void init(ServletConfig config)
      throws ServletException 
      {
        
      super.init(config);
      this.config = config;
      this.context = config.getServletContext();
      
      log("XqlServlet Initialized");
        
    }

    
    public void apply(CallContext callContext)
      {
        try
          {
          ServletCallContext ctx = (ServletCallContext) callContext;
          String xqlUri = ctx.request.getServletPath();
  
          log("XqlEngine --> "+xqlUri);
          log("\t     ServletPath: " +
                                    ctx.request.getServletPath());
          log("\t        PathInfo: " +
                                    ctx.request.getPathInfo());
          log("\t      RequestURI: " +
                                    ctx.request.getRequestURI());
          log("\t     QueryString: " +
                                    ctx.request.getQueryString());
          log("\t  Request Params: ");
          
          Enumeration e = ctx.request.getParameterNames();
          while (e.hasMoreElements()) 
            {
            String name = (String) e.nextElement();
            log("\t\t " + name + " = " + ctx.request.getParameter(name));
            } 
            serviceXqlFile(ctx, xqlUri);
          }
        catch (ServletException e)
          {
          log("Xql Interpreter Exception, because of ", e);
          }
        catch(IOException e) 
          { 
          log("Xql Interpreter Exception, because of ", e);
          }         
        catch(Throwable e) 
          { 
          log("Xql Interpreter Exception, because of ", e);
          }         
      }
 

    private void serviceXqlFile(ServletCallContext ctx, String xqlUri)
      throws ServletException, IOException, Throwable 
      {
      InputStream resourceStream = context.getResourceAsStream(xqlUri);
      if (resourceStream == null) 
        {
        ctx.response.sendError(HttpServletResponse.SC_NOT_FOUND, xqlUri);
        return;
        } 
      else 
        {
          runXql(ctx, new InputStreamReader(resourceStream));
          resourceStream.close();
        }
      }


    protected void runXql(ServletCallContext ctx, Reader xqlReader)
      throws Throwable
      {
        ServletOutputStream os = ctx.response.getOutputStream();
        try
        {
          XQuery xq = new XQuery();
          ByteArrayOutputStream aos = new ByteArrayOutputStream();
          XMLPrinter pp = new XMLPrinter( ctx.response.getOutputStream() );
          CallContext.setInstance( ctx );
          xq.eval(new InPort(xqlReader), (Writer)pp);
          pp.println();
          pp.flush();
          pp.close();
          os.write( aos.toByteArray() );
        }
        catch (Throwable t)
        {
          StringBuffer errorMessage = new StringBuffer();
          errorMessage.append("XQuery Compilation Errors:\n\n");
          errorMessage.append( t.getMessage() );
          os.print( errorMessage.toString() );
        }
        finally
        {
          os.flush();
        }
      }
    


    public void destroy() 
    {
      super.destroy();
    }


}
